@@ -6,7 +6,6 @@ import json
 import contextlib
 import datetime
 
-# from .n0struct_utils_compare import *
 from .n0struct_utils_compare import get__flag_compare_check_different_types
 from .n0struct_utils_compare import get__flag_compare_return_difference_of_values
 from .n0struct_utils_compare import get__flag_compare_return_equal
@@ -20,7 +19,6 @@ from .n0struct_utils_find import split_name_index
 from .n0struct_n0list_ import n0list_
 from .n0struct_n0dict_ import n0dict_
 
-from .n0struct_logging import *
 # ******************************************************************************
 # ******************************************************************************
 class n0list(n0list_):
@@ -68,11 +66,11 @@ class n0list(n0list_):
             return_lists,
             xpath_found_str: str = "/") \
         ->  typing.Tuple[
-                typing.Union[n0dict, n0list, None],
-                typing.Union[str, int, None],
-                typing.Union[typing.Any, None],
+                typing.Union[None, n0dict, n0list],
+                typing.Union[None, str, int],
+                typing.Union[None, typing.Any],
                 str,
-                typing.Union[list, None]
+                typing.Union[None, list]
             ]:
         """
         [0] = parent node: n0dict/n0list
@@ -106,7 +104,7 @@ class n0list(n0list_):
             if isinstance(node_index, str):
                 node_index_str = node_index
             else:
-                raise Exception("Impossible to have complex index for lists")
+                raise IndexError("Impossible to have complex index for lists")
 
             # ..................................................................
             # Try to check all [*] items in the loop
@@ -114,14 +112,7 @@ class n0list(n0list_):
             if node_index_str == "*":
                 cur_values = n0list()
                 fst_parent_node = fst_node_name_index = fst_value = fst_found_xpath_str = None
-                # n0debug("parent_node")
                 for i,next_parent_node in enumerate(parent_node):
-                    # n0debug("i")
-                    # n0debug("next_parent_node")
-                    # n0debug_calc(["[%d]" % i] + xpath_list[1:])
-                    # n0debug("return_lists")
-                    # n0debug("xpath_found_str")
-
                     if isinstance(next_parent_node, dict):
                         cur_parent_node, cur_node_name_index, cur_value, cur_found_xpath_str, \
                             cur_not_found_xpath_list = n0dict._find(next_parent_node, xpath_list[1:], next_parent_node, return_lists,  xpath_found_str + f"[{i}]")
@@ -129,13 +120,14 @@ class n0list(n0list_):
                         cur_parent_node, cur_node_name_index, cur_value, cur_found_xpath_str, \
                             cur_not_found_xpath_list = self._find(xpath_list[1:], next_parent_node, return_lists, xpath_found_str + f"[{i}]")
                     else:
-                        raise Exception("Unexpected type (%s) of %s" % (type(next_parent_node), str(next_parent_node)))
+                        raise TypeError("Unexpected type (%s) of %s" % (type(next_parent_node), str(next_parent_node)))
 
                     if not cur_not_found_xpath_list:
                         cur_values.append(cur_value)
                         if not fst_found_xpath_str:
                             fst_parent_node, fst_node_name_index, fst_value, fst_found_xpath_str = \
                                 cur_parent_node, cur_node_name_index, cur_value, cur_found_xpath_str
+
                 if not return_lists and len(cur_values) == 1:
                     cur_values = cur_values[0]
                 if fst_found_xpath_str:
@@ -174,7 +166,7 @@ class n0list(n0list_):
                     if isinstance(next_parent_node, (list, tuple)):
                         return self._find(xpath_list[1:], next_parent_node, return_lists, "%s[%d]" % (xpath_found_str, node_index_int))
                     else:
-                        raise Exception("Unexpected type (%s) of %s" % (type(next_parent_node), str(next_parent_node)))
+                        raise TypeError("Unexpected type (%s) of %s" % (type(next_parent_node), str(next_parent_node)))
     # **************************************************************************
     # * n0list. direct_compare(..): only n0dict. direct_compare/compare have one_of_list_compare
     # **************************************************************************
@@ -220,9 +212,9 @@ class n0list(n0list_):
                     transform
         """
         if continuity_check != "continuity_check":
-            raise Exception("Incorrect order of arguments")
+            raise SyntaxError("Incorrect order of arguments")
         if not isinstance(other, n0list):
-            raise Exception("n0list.direct_compare(): other (%s) must be n0list" % str(other))
+            raise TypeError("n0list.direct_compare(): other (%s) must be n0list" % str(other))
 
         result = n0dict({
             "differences":  [],
@@ -301,7 +293,6 @@ class n0list(n0list_):
                             )
                         )
                     else:
-                        # NOT TESTED! #1
                         if get__flag_compare_return_equal():
                             result["self_equal"].append(self)
                             result["other_equal"].append(other)
@@ -317,7 +308,6 @@ class n0list(n0list_):
                             exclude_xpaths=exclude_xpaths, transform=transform,
                         )
                     )
-                # elif isinstance(self[i], (n0dict, dict, OrderedDict)):
                 elif isinstance(self[i], dict):
                     result.update_extend(
                         n0dict(self[i]).direct_compare(
@@ -336,7 +326,7 @@ class n0list(n0list_):
                     # So both are None
                     pass
                 else:
-                    raise Exception(
+                    raise TypeError(
                         "Not expected type %s in %s[%d]/%s[%d]" %
                         (
                             type(self[i]),
@@ -448,9 +438,9 @@ class n0list(n0list_):
                     transform
         """
         if continuity_check != "continuity_check":
-            raise Exception("n0list. compare(..): incorrect order of arguments")
+            raise SyntaxError("n0list. compare(..): incorrect order of arguments")
         if not isinstance(other, n0list):
-            raise Exception("n0list. compare(..): other (%s) must be n0list" % str(other))
+            raise TypeError("n0list. compare(..): other (%s) must be n0list" % str(other))
         result = n0dict({
             "differences":  [],
             "not_equal":    [],
@@ -477,7 +467,7 @@ class n0list(n0list_):
             except ValueError:
                 other_i = None
             # if composite_key in other_composite_keys:
-            if not other_i is None:
+            if other_i is not None:
                 # other_i = notmutable__other_not_exist_in_self.index(composite_key)
                 # other_i = other_not_exist_in_self[other_composite_keys.index(composite_key)][1]
                 # --- TRANSFORM: START -------------------------------------
@@ -572,7 +562,7 @@ class n0list(n0list_):
                         # So both are None
                         pass
                     else:
-                        raise Exception(
+                        raise TypeError(
                             "Not expected type %s in %s[%d]/%s[%d]" %
                             (
                                 type(self[self_i]),
@@ -626,9 +616,6 @@ class n0list(n0list_):
                                 other_name, other_i, str(other[other_i])
                             )
                         )
-                # ######### if type(self[i]) == type(other[i]):
-                # self_not_exist_in_other.remove(composite_key)
-                # other_not_exist_in_self.remove(composite_key)
                 del self_not_exist_in_other[[itm[0] for itm in self_not_exist_in_other].index(composite_key)]
                 del other_not_exist_in_self[[itm[0] for itm in other_not_exist_in_self].index(composite_key)]
             # ######### if key in other_not_exist_in_self:
@@ -700,10 +687,10 @@ class n0dict(n0dict_):
             _file = kw.pop("file", None)
             if _file:
                 with open(_file, "rb") as in_filehandler:
-                    buffer = in_filehandler.read()
-                    if buffer[:3] == b'\xEF\xBB\xBF': # UTF-8 BOM (Byte Order Mark)
-                        buffer = buffer[3:]
-                    self.__init__(buffer.decode('utf-8').strip(), **kw)
+                    buffer_bytes = in_filehandler.read()
+                    if buffer_bytes[:3] == b'\xEF\xBB\xBF': # UTF-8 BOM (Byte Order Mark)
+                        buffer_bytes = buffer_bytes[3:]
+                    self.__init__(buffer_bytes.decode('utf-8').strip(), **kw)
             else:
                 super(n0dict, self).__init__(*args, **kw)
         elif len__args == 1:
@@ -743,7 +730,6 @@ class n0dict(n0dict_):
                 if (len(args[0]) % 2) == 0 and all(isinstance(itm, str) for itm in args[0][0::2]):
                     for key, value in zip(args[0][0::2],args[0][1::2]):
                         self.update({key: value})
-                    # return None
                 # [(key1, value1), (key2, value2), ..., (keyN, valueN)]
                 elif all(isinstance(itm, (tuple,list)) and len(itm) == 2 and isinstance(itm[0], str) for itm in args[0]):
                     for pair in args[0]:
@@ -786,9 +772,9 @@ class n0dict(n0dict_):
                         ] = (),
     ) -> n0dict:
         if continuity_check != "continuity_check":
-            raise Exception("n0dict. compare(..): incorrect order of arguments")
+            raise SyntaxError("n0dict. compare(..): incorrect order of arguments")
         if not isinstance(other, n0dict):
-            raise Exception("n0dict. compare(..): other ((%s)%s) must be n0dict" % (type(other), str(other)))
+            raise TypeError("n0dict. compare(..): other ((%s)%s) must be n0dict" % (type(other), str(other)))
         result = n0dict({
             "differences":  [],
             "not_equal":    [],
@@ -811,17 +797,13 @@ class n0dict(n0dict_):
         # #############################################################
         for key in self_keys:
             if key in other:
-                # fullxpath = prefix + "/" + key
-                fullxpath = "%s/%s" % (prefix, key)
-                # n0debug("fullxpath")
+                fullxpath = prefix + "/" + key
                 if not xpath_match(fullxpath, exclude_xpaths):
                     # --- TRANSFORM: START -------------------------------------
                     # Transform self and other linked values with function transform[]()
                     self_value = self[key]
                     other_value = other[key]
-                    # n0debug("transform")
                     transform_i = xpath_match(fullxpath, [itm[0] for itm in transform])
-                    # n0debug("transform_i")
                     if transform_i:
                         transform_i -= 1
                         transform_self = transform[transform_i][1]
@@ -834,7 +816,6 @@ class n0dict(n0dict_):
                     # --- TRANSFORM: END ---------------------------------------
                     if type(self_value) == type(other_value):
                         if isinstance(self_value, (str, int, float, datetime.date)):
-                            # if self_value != other_value:
                             if self_value != other_value \
                                 and (not compare_only or xpath_match(fullxpath, compare_only)):
                                 # VERY IMPORTANT TO SHOW ORIGINAL VALUES, NOT TRANSFORMED
@@ -873,7 +854,6 @@ class n0dict(n0dict_):
                                     exclude_xpaths=exclude_xpaths, transform=transform,
                                 )
                             )
-                        # elif isinstance(self[key], (dict, OrderedDict)):
                         elif isinstance(self[key], dict):
                             result.update_extend(
                                 n0dict(self[key]).compare(
@@ -887,11 +867,10 @@ class n0dict(n0dict_):
                                 )
                             )
                         elif self[key] is None:
-                            # type(self[key]) == type(other[key]) and self[key] is None
-                            # So both are None
+                            # Because of type(self[key]) == type(other[key]) and self[key] is None, so both are None
                             pass
                         else:
-                            raise Exception("Not expected type %s in %s[\"%s\"]" % (type(self[key]), key, self_name))
+                            raise TypeError("Not expected type %s in %s[\"%s\"]" % (type(self[key]), key, self_name))
                     else:
                         if not compare_only or xpath_match(fullxpath, compare_only):
                             is_still_equal = False
@@ -992,7 +971,7 @@ class n0dict(n0dict_):
             transform: tuple = (),  # ()|None|empty mean nothing to transform
     ) -> n0dict:
         if continuity_check != "continuity_check":
-            raise Exception("n0dict. direct_compare(..): incorrect order of arguments")
+            raise SyntaxError("n0dict. direct_compare(..): incorrect order of arguments")
         return self.compare(
             other,
             self_name, other_name, prefix,
@@ -1003,18 +982,17 @@ class n0dict(n0dict_):
     # **************************************************************************
     # n0dict _find
     # **************************************************************************
-    # def _find(self, xpath_list: list, parent_node, return_lists, xpath_found_str: str = "/") -> list:
     def _find(self,
             xpath_list: typing.Union[str, list],
             parent_node,
             return_lists,
             xpath_found_str: str = "/") \
         ->  typing.Tuple[
-                typing.Union[n0dict, n0list, None],
-                typing.Union[str, int, None],
-                typing.Union[typing.Any, None],
+                typing.Union[None, n0dict, n0list],
+                typing.Union[None, str, int],
+                typing.Union[typing.Any],
                 str,
-                typing.Union[list, None]
+                typing.Union[None, list]
             ]:
         """
         [0] = parent node: n0dict/n0list
@@ -1040,7 +1018,7 @@ class n0dict(n0dict_):
         node_name, node_index = split_name_index(xpath_list[0])
         if not node_name and not node_index:
             n0debug("xpath_list")
-            raise Exception("Empty node_name and node_index")
+            raise ValueError("Empty node_name and node_index")
         # ##########################################################################################
         # Key in n0dict
         # ##########################################################################################
@@ -1064,7 +1042,7 @@ class n0dict(n0dict_):
                         if isinstance(cur_node_index, str):
                             nxt_parent_node = cur_parent_node[n0eval(cur_node_index)]
                         else:
-                            raise Exception("If index is in '%s', then (%s)'%s' must be str" %
+                            raise TypeError("If index is in '%s', then (%s)'%s' must be str" %
                                                 (
                                                     cur_node_name_index,
                                                     type(cur_node_index),
@@ -1095,16 +1073,11 @@ class n0dict(n0dict_):
 
             # if not isinstance(parent_node, (dict, OrderedDict, n0dict)):
             if not isinstance(parent_node, dict):
-                # raise IndexError("If key '%s' is set then (%s)'%s' must be n0dict at '%s'" %
-                    # (node_name, type(parent_node), str(parent_node), xpath_found_str)
-                # )
-                # raise IndexError(f"If key '{node_name}' is set then ({type(parent_node)})'{str(parent_node)}' must be n0dict at '{xpath_found_str}'")
                 raise IndexError(f"Internal error: Parent node '{xpath_found_str}' of '{node_name}' is redefined to ({type(parent_node)})'{str(parent_node)}', but must be n0dict")
             # ..................................................................
             # Try to check all [*] items in the loop
             # ..................................................................
             if node_name == "*":
-                # cur_values = n0list([])
                 cur_values = n0list()
                 fst_parent_node = fst_node_name_index = fst_value = fst_found_xpath_str = None
                 for next_node_name in parent_node:
@@ -1138,7 +1111,7 @@ class n0dict(n0dict_):
             # ..................................................................
             # Check key in dictionary
             # ..................................................................
-            if not node_name in parent_node:
+            if node_name not in parent_node:
                 #--------------------------------
                 # NOT FOUND: Node name in n0dict
                 #--------------------------------
@@ -1159,33 +1132,23 @@ class n0dict(n0dict_):
             # Deeper
             #*******************************
             if node_index is None:
-                # # 0.51: NOT CORRECTLY FIXED
-                # if isinstance(parent_node[node_name], (dict, OrderedDict, n0dict)):
-                    return self._find(
-                                    xpath_list[1:],
-                                    parent_node[node_name],
-                                    return_lists,
-                                    xpath_found_str + '/' + node_name
-                    )
-                # # 0.51: NOT CORRECTLY FIXED
-                # else:
-                    # return parent_node, None, None, xpath_found_str, xpath_list # Impossible to go deeper
+                return self._find(
+                                xpath_list[1:],
+                                parent_node[node_name],
+                                return_lists,
+                                xpath_found_str + '/' + node_name
+                )
             else:
-                # # 0.51: NOT CORRECTLY FIXED
-                # if isinstance(parent_node[node_name], (list, tuple, n0list, dict, OrderedDict, n0dict)):
-                    return self._find(
-                                    [
-                                        "[%s%s'%s']" % (node_index[0], node_index[1], node_index[2]) \
-                                        if isinstance(node_index, tuple) else \
-                                        "[%s]" % str(node_index) # Because of mypy: error: Not all arguments converted during string formatting
-                                    ] + xpath_list[1:],
-                                    parent_node[node_name],
-                                    return_lists,
-                                    xpath_found_str + '/' + node_name
-                    )
-                # # 0.51: NOT CORRECTLY FIXED
-                # else:
-                    # return parent_node, None, None, xpath_found_str, xpath_list  # Impossible to go deeper
+                return self._find(
+                                [
+                                    "[%s%s'%s']" % (node_index[0], node_index[1], node_index[2]) \
+                                    if isinstance(node_index, tuple) else \
+                                    "[%s]" % str(node_index) # Because of mypy: error: Not all arguments converted during string formatting
+                                ] + xpath_list[1:],
+                                parent_node[node_name],
+                                return_lists,
+                                xpath_found_str + '/' + node_name
+                )
         # ##########################################################################################
         # Index in n0list (node_index is not None)
         # ##########################################################################################
@@ -1206,7 +1169,6 @@ class n0dict(n0dict_):
                 if not isinstance(parent_node, (list, tuple)):
                     # Hidden list. Convert single item into list
                     parent_node = [parent_node]
-                # cur_values = n0list([])
                 cur_values = n0list()
                 fst_parent_node = fst_node_name_index = fst_value = fst_found_xpath_str = None
                 for i,cur_node in enumerate(parent_node):
@@ -1244,12 +1206,12 @@ class n0dict(n0dict_):
                     elif node_index[1][1] == '~':
                         comparing_result = node_index[2] in parent_node  # expected_value
                     else:
-                        raise Exception ("Unknown comparing command in %s" % str(node_index))
+                        raise SyntaxError("Unknown comparing command in %s" % str(node_index))
 
                     if node_index[1][0] == '!':
                         comparing_result = not comparing_result
                     elif node_index[1][0] != '=' and node_index[1][0] != '~':
-                        raise Exception ("Unknown comparing command in %s" % str(node_index))
+                        raise SyntaxError("Unknown comparing command in %s" % str(node_index))
 
                     if comparing_result:
                         # *******************************
@@ -1268,12 +1230,11 @@ class n0dict(n0dict_):
                         # *******************************
                         return self._find(["[*]"] + xpath_list, parent_node, return_lists, xpath_found_str)
 
-                    # if not isinstance(parent_node, (dict, OrderedDict, n0dict)):
                     if not isinstance(parent_node, dict):
                         raise IndexError("If key '%s' is set then (%s)'%s' must be n0dict at '%s'" %
                             (node_index[0], type(parent_node), str(parent_node), xpath_found_str)
                         )
-                    if not node_index[0] in parent_node:
+                    if node_index[0] not in parent_node:
                         return parent_node, None, None, xpath_found_str, xpath_list
 
                     return self._find(
@@ -1302,7 +1263,6 @@ class n0dict(n0dict_):
                     # NOT FOUND: Element in n0list
                     #--------------------------------
                     return parent_node, "[%d]" % node_index_int, None, xpath_found_str, xpath_list
-                    # raise IndexError("If we are looking for element of list '%s[%s]', then parent node (%s)'%s' must be list" % (xpath_found_str, node_index, type(parent_node), str(parent_node)))
 
                 if len(xpath_list) == 1:
                     #================================
@@ -1326,7 +1286,6 @@ class n0dict(n0dict_):
 
         next_node_name, next_node_index = split_name_index(xpath_list[0])  # possible both could be NOT empty
 
-        # if not cur_node_index is None:
         if cur_node_index is None:
             ####################################################################
             # Parent is DICTIONARY
@@ -1342,20 +1301,17 @@ class n0dict(n0dict_):
                 pass
 
             if next_node_name:
-                if not next_node_name in parent_node:
+                if next_node_name not in parent_node:
                     # New node
                     if next_node_index:
                         # NEWFIX
                         if next_node_index != "new()" and next_node_index != "0":
-                            # raise Exception("Nonsence! Impossible to add %s[%s] to the list (%s)%s"
-                            #                 % (cur_node_name, cur_node_index, type(parent_node), str(parent_node)))
-                            raise Exception(f"Nonsence! Impossible to add {next_node_name}[{next_node_index}] to the {cur_node_name}"
+                            raise SyntaxError(f"Nonsence! Impossible to add {next_node_name}[{next_node_index}] to the {cur_node_name}"
                                             + (f"[{cur_node_index}]" if cur_node_index else "")
                                             + f" {str(parent_node)}"
                             )
                         parent_node.update({next_node_name: n0list([None])})
                         next_node = parent_node[next_node_name]
-                        # next_node_name_index = "[%s]" % next_node_index
                         next_node_name_index = "[last()]"
                         # item[0] == None, will be reused at the next step with last()
                     else:
@@ -1386,14 +1342,12 @@ class n0dict(n0dict_):
             # or (isinstance(parent_node, (list, tuple)) and n0eval(cur_node_index) == len(parent_node)) \
             # or (isinstance(parent_node[cur_node_name], (list, tuple)) and n0eval(cur_node_index) == len(parent_node)[cur_node_name]):
                 if cur_node_name and not isinstance(parent_node[cur_node_name], (list, tuple, n0list)):
-                # if not isinstance(parent_node[cur_node_name], (list, tuple, n0list)):
                     ####################################################################
                     # Convert not LIST into the list
                     ####################################################################
                     if len(parent_node[cur_node_name]):
                         parent_node.update({cur_node_name: n0list([parent_node[cur_node_name]])})
                     else:
-                        # parent_node.update({cur_node_name: n0list([])})
                         parent_node.update({cur_node_name: n0list()})
                     parent_node = parent_node[cur_node_name]
                 if next_node_name:
@@ -1404,7 +1358,6 @@ class n0dict(n0dict_):
                         next_node_name_index = next_node_name
                     else:
                         # Next is list under dict
-                        # parent_node.append(n0dict({next_node_name: n0list([])}))
                         parent_node.append(n0dict({next_node_name: n0list()}))
                         next_node = parent_node[-1][next_node_name]
                         # Expected to have 'new()' in next_node_index, else it will be failed at the next step
@@ -1416,11 +1369,11 @@ class n0dict(n0dict_):
                     # Expected to have 'new()' in next_node_index, else it will be failed at the next step
                     next_node_name_index = "[%s]" % str(next_node_index)
                 else:
-                    raise Exception("Nonsence! Both next_node_name and next_node_index could NOT be empty")
+                    raise ValueError("Nonsence! Both next_node_name and next_node_index could NOT be empty")
             elif cur_node_index == "last()":
                 # Came from previous level: we create [None] and point to [last()] for exchange
                 if not isinstance(parent_node, (list, tuple, n0list)):
-                    raise Exception("Nonsence! if index '%s' is set then (%s)%s must be n0list"
+                    raise ValueError("Nonsence! if index '%s' is set then (%s)%s must be n0list"
                                     % (cur_node_index, type(parent_node), str(parent_node)))
                 if next_node_name:
                     if not next_node_index:
@@ -1447,9 +1400,7 @@ class n0dict(n0dict_):
                 next_node = parent_node
                 next_node_name_index = "[last()]"
             else:
-                # raise Exception("Nonsence! Impossible to add %s[%s] to the list (%s)%s"
-                #                 % (cur_node_name, cur_node_index, type(parent_node), str(parent_node)))
-                raise Exception(f"Nonsence! Impossible to add item {cur_node_index} to the list {str(parent_node)}")
+                raise SyntaxError(f"Nonsence! Impossible to add item {cur_node_index} to the list {str(parent_node)}")
 
         if len(xpath_list) == 1:
             return next_node, next_node_name_index
@@ -1459,7 +1410,6 @@ class n0dict(n0dict_):
     def update(self, xpath: typing.Union[dict, str], new_value: str = None) -> n0dict__:
         # **********************************************************************
         def multi_define(xpath, new_value):
-            # if isinstance(new_value, (dict, OrderedDict, n0dict)):
             if isinstance(new_value, dict):
                 self[xpath] = n0dict(new_value, recursively=True)
             elif isinstance(new_value, (list, tuple)):
@@ -1470,10 +1420,10 @@ class n0dict(n0dict_):
         if isinstance(xpath, dict) and new_value is None:
             for item_key in xpath:
                 multi_define(item_key, xpath[item_key])
-        elif isinstance(xpath, str) and not new_value is None:
+        elif isinstance(xpath, str) and new_value is not None:
             multi_define(xpath, new_value)
         else:
-            raise Exception("Received (%s,%s) as argument, but expected (key,value) or (dict)." % (type(xpath), type(new_value)))
+            raise TypeError("Received (%s,%s) as argument, but expected (key,value) or (dict)." % (type(xpath), type(new_value)))
         return self
 # ******************************************************************************
 # ******************************************************************************
